#--------------------------------------------------------------------------------
# cbr-tb.tcl
# Author: Jeremy Ethridge.
# Dates: June 29-July 5, 1999.
# Notes: A DS-RED script that uses CBR traffic agents and the Token Bucket policer.
#
#    ----
#    |s1|-----------
#    ----   10 Mb   \
#            5 ms    \
#                     \----           ------          ----           ------
#                      |e1|-----------|core|----------|e2|-----------|dest|
#                     /----   10 Mb   ------   5 Mb   ----   10 Mb   ------
#                    /         5 ms            5 ms           5 ms
#    ----           /
#    |s2|-----------
#    ----   10 Mb
#            5 ms
#
#--------------------------------------------------------------------------------


set ns [new Simulator]

set cir0  1000000
set cbs0    10000
set rate0 4000000
set cir1  2000000
set cbs1    15000
set rate1 4000000

set testTime 85.0
set packetSize 1000


# Set up the network topology shown at the top of this file:
set s1 [$ns node]
set s2 [$ns node]
set e1 [$ns node]
set core [$ns node]
set e2 [$ns node]
set dest [$ns node]

$ns duplex-link $s1 $e1 10Mb 5ms DropTail
$ns duplex-link $s2 $e1 10Mb 5ms DropTail

$ns simplex-link $e1 $core 10Mb 5ms dsRED/edge
$ns simplex-link $core $e1 10Mb 5ms dsRED/core

$ns simplex-link $core $e2 5Mb 5ms dsRED/core
$ns simplex-link $e2 $core 5Mb 5ms dsRED/edge

$ns duplex-link $e2 $dest 10Mb 5ms DropTail


$ns duplex-link-op $s1 $e1 orient down-right
$ns duplex-link-op $s2 $e1 orient up-right
$ns duplex-link-op $e1 $core orient right
$ns duplex-link-op $core $e2 orient right
$ns duplex-link-op $e2 $dest orient right


set qE1C [[$ns link $e1 $core] queue]
set qE2C [[$ns link $e2 $core] queue]
set qCE1 [[$ns link $core $e1] queue]
set qCE2 [[$ns link $core $e2] queue]


# Set DS RED parameters from Edge1 to Core:
$qE1C meanPktSize $packetSize
$qE1C set numQueues_ 1
$qE1C setNumPrec 2
$qE1C addPolicyEntry [$s1 id] [$dest id] TokenBucket 20 $cir0 $cbs0
$qE1C addPolicyEntry [$s2 id] [$dest id] TokenBucket 10 $cir1 $cbs1
$qE1C addPolicerEntry TokenBucket 10 11
$qE1C addPolicerEntry TokenBucket 20 21
$qE1C addPHBEntry 10 0 0
$qE1C addPHBEntry 11 0 1
$qE1C addPHBEntry 20 0 0
$qE1C addPHBEntry 21 0 1
$qE1C configQ 0 0 20 40 0.02
$qE1C configQ 0 1 10 20 0.10

puts "================> Testing getMeter:"
puts "[$qE1C getMeter [$s1 id] [$dest id]]"
puts "[$qE1C getMeter [$s2 id] [$dest id]]"
puts "================> Before removing:"
$qE1C printPolicyTable
$qE1C printPolicerTable
puts "\n================> Remove the entry [$s2 id] [$dest id]:"
$qE1C delPolicyEntry [$s2 id] [$dest id]
$qE1C printPolicyTable
$qE1C printPolicerTable
puts "\n================> Put it back:"
$qE1C addPolicyEntry [$s2 id] [$dest id] TokenBucket 10 $cir1 $cbs1
$qE1C printPolicyTable
$qE1C printPolicerTable
puts "\n================> Remove the entry [$s1 id] [$dest id]:"
$qE1C delPolicyEntry [$s1 id] [$dest id]
$qE1C printPolicyTable
$qE1C printPolicerTable
puts "\n================> Put it back:"
$qE1C addPolicyEntry [$s1 id] [$dest id] TokenBucket 20 $cir0 $cbs0
$qE1C printPolicyTable
$qE1C printPolicerTable
puts "\n================> Remove both:"
$qE1C delPolicyEntry [$s1 id] [$dest id]
$qE1C delPolicyEntry [$s2 id] [$dest id]
$qE1C printPolicyTable
$qE1C printPolicerTable
puts "\nBye!"

